;---------------------------------------------------------------------------------
;                                                                                ;
; test harness and implementation of a new cycle exact hard-restart              ;
;                                                                                ;
; implementation by shrydar/ChristopherJam, feat. ideas by Linus 'lft' Åkesson   ;
;                                                                                ;
;---------------------------------------------------------------------------------

.macro waitNy ncycles   ; clobbers Y if ncycles>7
	.if ncycles=0
	.exitmacro
	.elseif ncycles=2 || ncycles=4 || ncycles=6
	nop
	waitNy (ncycles-2)
	.elseif ncycles=3 || ncycles=5 || ncycles=7
	jmp :+
:	
	waitNy (ncycles-3)
	.else
	ldy#(ncycles-3)/5     ;code waits 5n+1
:	dey
	bne :-
	waitNy (ncycles-(ncycles-3)/5*5-1)
	.endif
.endmacro


	.bss
fc:	.res 1


v3    = $d40e
v3CR  = v3+4
v3AD  = v3+5
v3SR  = v3+6
v3env = $d41c



	.segment "STARTUP"
	.word basicstub		; load address

basicstub:
	.byte $0b,$08,$0a,$00,$9e,$32,$30,$36,$31,0,0,0 ; sys 2061
	jmp init

	.data
message:
	.byt 147,159,14   ; clear, cyan, lower
	.byt "aFTER TRAD. HARD RESET:",13,13,13
	.byt "aFTER STABILISErc:",0

	.code
init:
	sei
:	bit $d012
	bmi :-
:	bit $d012
	bpl :-

	lda#$0b
	sta $d011
	lda#4
	sta 646
	sta $d021

	ldx#0
	lda message
:	jsr $ffd2
	inx
	lda message,x
	bne :-
	;lda#$16
	;sta $d018


	lda#0
	sta $d020
	sta $d021
qq:
	lda#$0b
	sta $d011
	jsr envtest
	lda#$1b
	sta $d011
	jsr wframe  ; time to see last result pair, and also HR for next comparison
	jsr wframe
	jmp qq

	cli
	rts

wframe:
:	bit $d012
	bmi :-
:	bit $d012
	bpl :-
	rts

.macro linetest rowptr
	ldx #1
	stx v3CR
	ldx#0
:	lda v3env
	sta rowptr,x
	inx
	cpx#20
	bne :-

	lda#$10
	sta v3AD
	waitNy 20
	lda#$00     ; try changing back to a faster rate - unsafe if we're not stably at a known RC!
	sta v3AD

:	lda v3env
	sta rowptr,x
	inx
	cpx#40
	bne :-
.endmacro

envtest:
	jsr randelay
	sei

	linetest $0428
	jsr stabiliseRC3
	linetest $04a0

	lda#0
	sta v3AD
	sta v3SR
	sta v3CR
	rts

randelay:
	inc fc
	lda fc
	ror
	ror
	eor fc
	ror
	adc#1
	tay
:	dex
	bne :-
	ldx#1
	dey
	bne :-
	rts

;------------------------------------------------------------------------------
; stabiliseRC3                                                                ;
;                                                                             ;
;    place SID in a known state, with quiescent env3 and ADSR=0000            ;
;                                                                             ;
;    All DMA and interrupts must be disabled for the last 600 cycles          ;
;                                                                             ;
;------------------------------------------------------------------------------

	.align 256

stabiliseRC3:
	lda#$00
	sta v3AD
	sta v3CR
	lda#$f0
	sta v3SR

	ldy#25        ; wait >0x7fff cycles to recapture
	ldx#128
:	dex
	bne :-
	dey
	bpl :-

	lda#$01
	sta v3CR       ; start rise to $ff

	ldx#205        ; wait >256*9 cycles for rise to maximum
:	dex
	bne :-
:	dex
	bne :-
	               ; from this point on, timing is critical
	lda#$02
	ldx#$01
	ldy#$00
.repeat 7          ; here's where the magic happens.  Potential RC values are released
	sta   v3AD     ; one at a time into a 63 cycle bottle, at intervals multiples of
	jsr wait20     ; 9 cycles apart
	stx   v3AD
	nop
	nop
	sty   v3AD
.endrep
	sta   v3AD
	waitNy 18      ; wait 18 cycles (clobbers Y, as ncycles>7)

	lda#$44        ; rate 4, limit 149
	stx v3AD
	sta v3AD
	sta v3SR
	               ; rate counter should now be one of 9 different values that
	               ; are all equal modulo 9, in the range 0 to 99
	lda#$00
	sta v3CR       ; ADSR=$4444, env=$ff, switching to release

	waitNy 170     ; wait for env to drop to $fe
	ldx#$01        ; request attack
	stx v3CR       ; we've now a few cycles grace before env reaches ff in which to switch ADSR to $40f0 

	lda#$40
	sta v3AD
	lda#$f0
	sta v3SR
	waitNy 99      ; wait for the entire packet of potential values to be captured into the 9 cycle decay limit loop
	               ; now ADSR = 40f0, RC is synchronised, env=$ff 

	lda #$11       ; next we need to force overflow
	sta v3AD       ; by switching to decay, then back to attack before env drops below $ff
	lda#$f1        ; we do this at a rate of 1 (RC limit of 31) to give us time for register fiddling.
	sta v3SR
	ldx #0
	stx v3CR       ; drop into decay

	ldx #1
	stx v3CR       ; return to attack - this'll increase env to $00
	waitNy 5
	lda#$00        ; recapture to the fastest rate
	sta v3AD       ; this write must be performed while RC<9, or we'll trigger the bug again.
	sta v3SR
	sta v3CR       ; and drop back to release state
	rts

wait20:
	nop
	nop
	nop
	nop
	rts


